<?php

declare(strict_types=1);

use PrestaShop\Module\LCVPrestaConnector\BridgeAdapter;
use PrestaShop\Module\LCVPrestaConnector\MultipleProductsManager;
use PrestaShop\Module\LCVPrestaConnector\SupplierRefManager;
use PrestaShop\Module\LCVPrestaConnector\SyncConfiguration;
use PrestaShop\Module\LCVPrestaConnector\Syncer;
use PrestaShop\PrestaShop\Adapter\ServiceLocator;
use PrestaShop\PrestaShop\Adapter\SymfonyContainer;
use PrestaShopBundle\Entity\Repository\TabRepository;

if (!defined('_PS_VERSION_'))
  exit;


class LCVPrestaConnector extends Module
{
    /**
     * Identifiant du module
     */
    public string $BridgePrefix;

      /**
     * chemin vers tpl
     */
    public string $TplPath;

    /**
     * Préfixe complet des tables
     */
    public string $TblPrefix;

    /**
     * Préfixe court des tables
     */
    public string $TblShortPrefix;

     /**
     * Configuration du module
     *      
     */
    private ?SyncConfiguration $cfg = null;

    /**
     * Lien vers le pont
     */
    private ?BridgeAdapter $bridge = null;

    /**
     * Gestionnaire de produits connexes
     */
    private ?MultipleProductsManager $mpm = null;
    

     /**
     * New instance
     * 
     */
    public function __construct()
    {
        // Nom du module
        $this->name = basename(__FILE__, '.php');

        // On va se noter une constante pour identifier la version de notre module
        // Cette constante, nommée Id est composée des 3 premieres lettres du nom du module
        $this->BridgePrefix = substr($this->name, 0, 3);
        if ($this->BridgePrefix == 'pre')
        {
            // Nous sommes en développement, on va prendre le contenu du fichier local 'DEV'
            // Si ce dernier n'existe pas, on le crée avec 'pol' à l'intérieur
            // Puis on charge le prefixe avec le contenu du fichier (nettoyer des espaces et des retours chariots)
            $file = __DIR__ . '/DEV';
            if (!file_exists($file))
                file_put_contents($file, 'pol');
            $this->BridgePrefix = trim(file_get_contents($file));            
        }

        $this->TblPrefix = _DB_PREFIX_ . 'pc_' . $this->BridgePrefix . '_';
        $this->TblShortPrefix = 'pc_' . $this->BridgePrefix . '_';
        $this->TplPath = '@Modules/' . $this->name . '/views/templates/';

        $this->author = 'LCV';
                
        $this->version = '8.0.6';

        $this->tab = 'synchronisation';        
        $this->need_instance = 0;
        $this->ps_versions_compliancy = [
            'min' => '8.0.0',
            'max' => _PS_VERSION_,
        ];
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->trans('LCVPrestaConnector', [], 'Modules.prestaconnector.Admin');
        $this->description = $this->trans('This module provide sync between your Prestashop and your backoffice © LCV.', [], 'Modules.prestaconnector.Admin');

        // On enrichi la description avec le changelog ici
        $this->description = "<h3 id=\"version-8.1.0\">Version 8.1.0</h3> <ul> <li>Possibilité de gérer des poids par défaut par catégories de produits.</li> <li>Importation des modifications de commandes, des remboursements et des retours sur le backoffice</li> </ul> <h3 id=\"version-8.0.4\">Version 8.0.4</h3> <p>Version corrective.</p> <ul> <li>Possibilité de tirer les modifications produits depuis la fiche d’un produit.</li> <li>Optimisation du tirage des soldes // INJECT_CHANGELOGamp; promos en même temps que les prix produits.</li> </ul> <h3 id=\"version-8.0.3\">Version 8.0.3</h3> <p>Version de base permettant de synchroniser votre catalogue, vos stocks et vos prix entre votre backoffice et votre site Internet.</p>";

        $this->confirmUninstall = $this->trans('Are you sure you want to uninstall this module ?', [], 'Modules.Polarisprestaconnector.Admin');        

        // Auto-connexion au backoffice
        if (isset($_REQUEST["vega-auto-connect"]) && $_REQUEST["vega-auto-connect"] == "yes")
            $this->autoConnect();
    }

    /**
     * Obtenir les hooks utilisés par ce module
     */
    public function getHooks()
	{
		$hooks = array(
            'displayHeader',
			'dashboardZoneOne',
			'displayBackOfficeHeader',
            'actionGetAdminOrderButtons',
            'displayMultiProductSelector',
			);

		return $hooks;
	}

    /**
     * Obtenir les tabs utilisés par ce module
     */
    public function getTabs()
    {
        $backofficeId = $this->getBridge()->getId();
        $tabs = [];

        $name = [];
        foreach (Language::getLanguages() as $lang)
            $name[$lang['id_lang']] = $this->trans('Mapping', [], 'Modules.'.$this->name.'.Admin', $lang['locale'])
                .' '.$this->getBridge()->getId();

        $tabs[] = [ 'class_name' => 'Mappage' . $backofficeId,
                    'route_name' => $this->name . '_map_category',
                    'parent_class' => 'AdminCatalog',
                    'name' => $name,
                    ];

        $name = [];
        foreach (Language::getLanguages() as $lang)
            $name[$lang['id_lang']] = $this->trans('Audit', [], 'Modules.'.$this->name.'.Admin', $lang['locale'])
                .' '.$this->getBridge()->getId();

        $tabs[] = [ 'class_name' => 'Audit'.$backofficeId,
                    'route_name' => $this->name . '_audit',
                    'parent_class' => 'AdminAdvancedParameters',
                    'name' => $name,
                    ];

        return $tabs;
    }

    /**
     * Auto-connexion au backoffice avec identification auprès du serveur de l'éditeur
     */
    public function autoConnect()
    {
        $errMsg = null;
        try
        {
            $uid = isset($_REQUEST["uid"]) ? $_REQUEST["uid"] : '';
            $token = isset($_REQUEST["token"]) ? $_REQUEST["token"] : '';

            if (!empty($uid) & !empty($token))
            {
                // On va lancer une requête CURL sur https://extranet.vega-info.fr/manager/$uid/$token
                // pour vérifier les informations de connexion
                // attention à bien encoder uid et token pour les passer en paramètres de l'URL
                $url = "https://extranet.vega-info.fr/manager/api/Public/MaintenanceAccess/".urlencode($uid)."/".urlencode($token);

                $ch = curl_init($url);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
                curl_setopt($ch, CURLOPT_TIMEOUT, 5);
                $result = curl_exec($ch);
                $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                curl_close($ch);
                if ($http_code == 200)
                {
                    $result = json_decode($result, true);
                    if (count($result))
                    {
                        // C'est bon, on a une réponse !
                        // On doit donc se connecter ici
                        $admin_mail = "admin@vega-info.fr";
			            $admin_path = "";			

                        // Fabrication d'un mot de passe aléatoire
                        $admin_pass = Tools::passwdGen(24);

                        // Recherche du chemin du répertoire admin ...
                        foreach (glob(_PS_ROOT_DIR_."/admin*") as $file)
                        {
                            if (is_dir($file))
                                $admin_path = basename($file)."/";
                        }

                        // Remplacement du code de maintenance ?                        
                        $employee = (new Employee())->getByEmail($admin_mail);
                        if (!$employee || empty($employee->id))
                        {
                            // Création de l'employee
                            $employee = new Employee();
                            $employee->firstname = "Service technique";
                            $employee->lastname = "LCV";
                            $employee->email = $admin_mail;
                            $employee->id_lang = Configuration::get('PS_LANG_DEFAULT');	
                        }

                        // Ensuite, on force le mot de passe                        
                        $employee->setWsPasswd($admin_pass);
                        $employee->id_profile = _PS_ADMIN_PROFILE_;
                        $employee->active = true;
                        $employee->save();

                        header('HTTP/1.0 200 OK');
                        include (__DIR__ . '/src/auto-connect.php');
                        ob_flush();
                        die();
                    }
                    else
                        throw new \Exception("Le serveur d'authentification a renvoyé une réponse vide !");
                }
                else
                    throw new \Exception("Le serveur d'authentification a renvoyé un code d'erreur ".$http_code." !");
            }
            else
                throw new \Exception("UID et/ou token manquants !");
        }
        catch (\Exception $e)
        {
            $errMsg = $e->getMessage();
        }

        // Si on arrive ici, c'est que l'auto-connexion a échoué
        // On répond un 403 forbidden
        header('HTTP/1.0 403 Forbidden');
        if ($errMsg)
            echo $errMsg;
        ob_flush();
        die();
    }

    /**
     * Réenregistrement des tabs...
     */
    public function registerTabs()
    {
        // Attention, on a pas de container symfony sur un contrôleur v7
        $container = SymfonyContainer::getInstance();
        $tabRepository = $container ? $container->get('prestashop.core.admin.tab.repository') : null;
        
        foreach ($this->getTabs() as $tabData) {
            // Vérifier si le tab existe déjà
            $exists = $tabRepository ?
                        $tabRepository->findOneIdByClassName($tabData['class_name']) :
                        Tab::getIdFromClassName($tabData['class_name']);

            if (!$exists) {
                $tab = new Tab();
                $tab->active = 1;
                $tab->class_name = $tabData['class_name'];
                $tab->route_name = $tabData['route_name'];
                if (isset($tabData['parent_class']))
                    $tab->id_parent = $tabRepository ?
                                            $tabRepository->findOneIdByClassName($tabData['parent_class']) :
                                            Tab::getIdFromClassName($tabData['parent_class']);                
                $tab->module = $this->name;                
                $tab->name = $tabData['name'];

                // Sauvegarder le tab
                $tab->save();
            }
        }
    }

    /**
     * Déclenche une mise à jour de la base si nécessaire !
     */
    public function dbUpgrade()
    {       
        // Faut-il déclencher la mise à jour de la base de données du module ?
        // pour le savoir on regarde, si la version enregistrée dans la configuration est inférieure à la version actuelle                
        $currentDbVersion = \Configuration::get($this->BridgePrefix . 'prestaconnector_version');
        if (!$currentDbVersion)
            $currentDbVersion = '0.0';

        if (version_compare($currentDbVersion, $this->version) < 0) {        
            // On met à jour la version du module
            require_once _PS_MODULE_DIR_ . $this->name . '/db/upgrade.php';
            $upgrade = new \PrestaShop\Module\LCVPrestaConnector\Upgrade();
            $upgrade->upgradeModule($this, $currentDbVersion);
        }
    }
    
    /*
		Réenregistrement des hooks...
	*/
	public function registerHooks($updatePosition = true)
	{
        // Pour chacun de nos hooks, s'ils ne sont pas déjà enregistrés, on les enregistre
		foreach ($this->getHooks() as $hook)
        {
            if (!$this->isRegisteredInHook($hook))
                $this->registerHook($hook);
        }

		if ($updatePosition)
			$this->updatePosition(Hook::getIdByName('dashboardZoneOne'), 0, 1);
	}

    /**
     * Obtenir l'objet de configuration
     */
    public function getCfg() : SyncConfiguration
    {
        if (!$this->cfg)
            $this->cfg = new SyncConfiguration($this);

        return $this->cfg;
    }

    /**
     * Obtenir le gestionnaire de produits connexes
     */
    public function getMpm() : MultipleProductsManager
    {
        if (!$this->mpm)
            $this->mpm = new MultipleProductsManager($this);

        return $this->mpm;
    }

    /**
     * Obtient le pont configuré
     */
    public function getBridge() : BridgeAdapter
    {
        if (!$this->bridge)
        {
            $classname = "PrestaShop\\Module\\LCVPrestaConnector\\" . $this->BridgePrefix . "\\Bridge";

            $this->bridge = new $classname($this);
        }
        return $this->bridge;
    }

    /*
    * Page de configuration
    */
    public function getContent()
    {
        Tools::redirectAdmin(
            SymfonyContainer::getInstance()->get('router')->generate($this->name . '_changelog')
        );
    }

    /**
     * Dashboard !
     */
    public function hookDashboardZoneOne($params)
	{
        $router = SymfonyContainer::getInstance()->get('router');
        $tpl_variables = array(
            'title_name' => 'Backoffice ' . $this->getBridge()->getId(),
            'module_name' => $this->name,
            'module_version' => $this->version,
            'configUrl'=> $router->generate($this->name . "_configuration"),
            'auditUrl' => $router->generate($this->name . "_audit"),
            'logsUrl' => $this->context->link->getAdminLink("AdminLogs"),
            'progress' => json_encode(Syncer::getSyncProgress($this)->toArray()),
            // On montre la configuration que si l'utilisateur en cours a une adresse mail qui se finit par @vega-info.fr ou @lcv.fr
            'show_cfg' => preg_match('/@(vega-info|lcv)\.fr$/', $this->context->employee->email),
        );

        return $this->get('twig')->render($this->TplPath . 'hook/dashboard.html.twig', $tpl_variables);
	}

    /**
     * Injectons du CSS
     */
    public function hookDisplayHeader()
    {
        // Ajouter le fichier CSS à la page du front-office
        $this->context->controller->addCSS($this->_path . 'views/css/front.css');
    }

    /**
     * Injection du JS et CSS dans l'admin
     * 
     */
    public function hookDisplayBackOfficeHeader($params)
    {
        $this->context->controller->addJS($this->_path.'views/js/common.js');
        $this->context->controller->addJS($this->_path.'views/js/admin.js');
        $this->context->controller->addCSS($this->_path.'views/css/admin.css');

        // Si nous sommes sur l'admin controller, alors on tweak l'interface
        switch (Tools::getValue('controller'))
        {
            case "AdminProducts":
                return $this->tweakAdminProductUI($params);
        }

        return '';
    }

    /**
     * Contrôle des champs protégés
     * 
     * @param array $params
     *
     * @return string
     */
    public function tweakAdminProductUI($params) : string
    {
        // Si nous sommes en mode sans synchro de fiche, inutile de protéger les champs, donc on sort
        $cfg = $this->getCfg();
        if ($cfg->get(SyncConfiguration::CFG_WORKMODE) != 1)
            return '';

        $syncCfg = $cfg->get(SyncConfiguration::CFG_DATA_SYNC);
        if (empty($syncCfg))
            return '';

        // Sinon, on va chercher à savoir si le produit est un produit qu'on entretien. 
        // Si nous retrouvons des références fournisseurs pour ce produit, alors c'est le cas
        $id_product = Tools::getValue('id_product');
		if (empty($id_product))
			return '';

        // On va chercher les références fournisseurs
        $refMngr = new SupplierRefManager($this->getCfg()->get(SyncConfiguration::CFG_SUPPLIER), [$id_product]);
        if ($refMngr->getNbProducts() == 0)
            return '';    // Non pas géré

        $locked = $this->getCfg()->isProductLocked((int) $id_product);

        $class = $this->name . "-disabled";
        $polarisColor = "#e4a12d";
		$style = "";
		$js = "";

        if (!$locked)
        {
            $js .= "$('.product-type-preview').before('<p class=\"".$class."\" >(Les champs de cette couleur sont contrôlés par votre backoffice ".$this->getBridge()->getId()." et <strong>vous ne devez pas les modifier</strong> directement sous Prestashop)</p>');\n";

            $item_to_disable = [
                '.product-type-preview',        // Pas de changement de type de produit
                '#combination-list-actions',    // ni de création de déclinaisons                        
            ];

            $fields_to_disable = [
                '#product_pricing_retail_price_price_tax_included', // Pas d'actions sur les prix
                '#product_pricing_retail_price_price_tax_excluded',            
            ];

            $fields_to_style = [
                '.delete-combination-item',     // pas de suppression de déclinaisons            
                '.edit-combination-item',       // pas de modification de déclinaisons
            ];

            // Cas particulier des caractéristiques
            $features = $this->getCfg()->get(SyncConfiguration::CFG_PR_ATTRIBUTES_MAPPING);
            $caracts = [];
            $id_lang = $this->context->language->id;
            if (!empty($features) && isset($features[SyncConfiguration::PRODUCT_ATTR_FEATURE]))
            {            
                foreach ($features[SyncConfiguration::PRODUCT_ATTR_FEATURE] as $feature => $id_feature)
                {
                    if ($id_feature > 0)
                    {
                        $feature = new Feature($id_feature);
                        $caracts[] = mb_strtolower($feature->name[$id_lang]);
                    }
                }
            }
            
            if (!empty($caracts))
                $js .= "$('#product_details_features_feature_values').before('<p class=\"".$class."\">Attention, certaines caractéristiques (".join(", ", $caracts).") sont maintenues par le module ".$this->name.".<br>Ne modifiez pas ces caractéristiques, le module annulera vos modifications pour ces catégories.</p>');\n";


            // Application des données de synchronisation
            foreach( [
                SyncConfiguration::CFG_PRODUCT_SYNC_REF => '#product_details_references_reference',
                SyncConfiguration::CFG_PRODUCT_SYNC_NAME => '#product_header_name',
                SyncConfiguration::CFG_PRODUCT_SYNC_DESCRIPTION => '#product_description_description',
                SyncConfiguration::CFG_PRODUCT_SYNC_MANUFACTURER => '#product_description_manufacturer',
                SyncConfiguration::CFG_PRODUCT_SYNC_DIMENSIONS => ['#product_shipping_dimensions_width', '#product_shipping_dimensions_height', '#product_shipping_dimensions_depth'],
                SyncConfiguration::CFG_PRODUCT_SYNC_WEIGHT => '#product_shipping_dimensions_weight',
                //SyncConfiguration::CFG_PRODUCT_SYNC_ECOTAX => true,
                //SyncConfiguration::CFG_PRODUCT_SYNC_LOCATION => '#combination_form_stock_options_stock_location',
                SyncConfiguration::CFG_PRODUCT_SYNC_TAXRATE => '#product_pricing_retail_price_tax_rules_group_id',
                SyncConfiguration::CFG_PRODUCT_SYNC_EAN13 => '#product_details_references_ean_13',            
                // SyncConfiguration::CFG_PRODUCT_SYNC_CATEGORIES => '#product_description_categories', => LAISSONS la possibilité au marchand de compléter la fiche comme il le souhaite
                SyncConfiguration::CFG_PRODUCT_SYNC_PHOTOS => '#product_description_images',            
            ] as $k => $v)
            {
                if (isset($syncCfg[$k]) && $syncCfg[$k])
                {
                    if (is_array($v))
                        $fields_to_disable = array_merge($fields_to_disable, $v);
                    else
                        $fields_to_disable[] = $v;
                }
            }

            // Application
            foreach ($item_to_disable as $field)
                $js .= "$('".$field."').attr('tabindex', '-1');\n$('".$field."').addClass('".$class."');\n";

            foreach ($fields_to_disable as $field)
                $js .= "$('".$field."').closest('.form-group').addClass('".$class."');\n"; // Remonter le champs pour mettre la classe sur le premier form-group qui englobe notre classe !            

            // Pour les fields_to_style, on laisse la possibilité tout de même de les modifier, mais on les met en couleur
            foreach ($fields_to_style as $field)
                $style .= $field." { color: $polarisColor !important; border-color: $polarisColor !important;}\n";

            $style .= ".".$class." { pointer-events: none !important; color: $polarisColor !important; border-color: $polarisColor !important;}\n";
            $style .= ".".$class." * { pointer-events: none !important; color: $polarisColor !important; border-color: $polarisColor !important;}\n";
            //$style .= join(", ", $fields_to_hide)." { display: none !important; }\n";
        }

        // Si le produit est verrouillé, on propose la possibilité de le déverrouiller
        if ($locked)
            $js .= '$("#product__toolbar_buttons").prepend(\'<a id="'.$this->name.'_lock" class="toolbar-button btn" style="color: crimson;" title="Ce produit est verrouillé. Il ne sera pas synchronisé depuis '. $this->getBridge()->getId() .'"><i class="material-icons">lock</i><span class="btn-label"></span></a>\');';
        else
        {            
            // Injection d'un bouton de synchronisation sur la fiche produit
            if ($this->getBridge()->canSyncOneProductOnly())
                $js .= '$("#product__toolbar_buttons").prepend(\'<a id="'.$this->name.'_sync" class="toolbar-button btn" style="color: #6c868e;" title="Synchroniser depuis ' . $this->getBridge()->getId() . '"><i class="material-icons">sync</i><span class="btn-label"></span></a>\');';
            $js .= '$("#product__toolbar_buttons").prepend(\'<a id="'.$this->name.'_lock" class="toolbar-button btn" style="color: #6c868e;" title="Empêcher de mettre à jour la fiche depuis '. $this->getBridge()->getId() .'"><i class="material-icons">lock_open</i><span class="btn-label"></span></a>\');';
            $js .= '$("#'.$this->name.'_sync").click((e) => ' . $this->name . '_syncProduct(e, '.$id_product.'));';        
        }

        // Synchroniser les données        
        $js .= '$("#'.$this->name.'_lock").click((e) => ' . $this->name . '_toggleLockProduct(e, '.$id_product.'));';

        $output = "";
        if (!empty($style))
			$output .= "<style>\n".$style."</style>\n";
		if (!empty($js))
			$output .= "<script language='javascript'>\n$(document).ready(function() {".$js."});\n</script>\n";

		return $output;
    }

    /**
	*	On agrémente le panneau d'administration des commandes avec nos fonctions.
    *
    *    @param array $params
	*/
	public function hookActionGetAdminOrderButtons(array $params)
	{
        $id_order = (int) $params['id_order'];
        $bar = $params['actions_bar_buttons_collection'];
        if ($bar && $id_order)
        {
            $psOrder = new \Order($id_order);
            if ($psOrder->id)
            {
        	    $bar->add(new \PrestaShop\PrestaShop\Core\Action\ActionsBarButton(
                     'btn-action', 
					['onclick' => $this->name.'_exportOrder(this, \'' . $psOrder->reference . '\')'], 
					'Réexporter vers '.$this->getBridge()->getId(),
            	));
            }
		}

		return "";
	}

    /**
     * Affichage du produit connexe manuel
     * 
     * @param array $params
     * @return string
     */
	public function hookDisplayMultiProductSelector($params) : string
	{
        if (!isset($params["product"]))
            return '⚠ Renseignez le paramètre `product` pour afficher le connecteur ici ! ⚠';   // Pas de chance

        if (!$params["product"]->id_product)
            return '';   // Pas de chance

        // On a besoin d'un gestionnaire de produits connexes
        $mpm = $this->getMpm();
        return $mpm->buildSelector($params["product"]->id_product);
	}

}